/****
Library for Raspberry Pi

Header file.

:Version:   2014-03-06

:Author:    Advamation, Roland Koebler (support@advamation.de)
:Copyright: (c) Advamation (info@advamation.de)
:License:   open-source / MIT

:RCS:       $Id$
***/

#ifndef __AM_RPI_H
#define __AM_RPI_H

#include <errno.h>
#include <stdint.h>
#include "advamation.h"

#define __AM_RPI_USE_CSI_GPIO 0     // use GPIOs on CSI-connector?

//----------------------------------------
// library init/exit/logging

AM_ERROR am_rpi_init(void);
void     am_rpi_exit(void);
AM_ERROR am_rpi_logging(enum am_log_level level, char const *logfile, enum am_log_level logfile_level);

//----------------------------------------
// system information
AM_ERROR am_rpi_info(char const **model, char const **rev, char const **manufacturer, uint32_t *rev_raw, uint64_t *serial);
AM_ERROR am_rpi_info_rev(uint32_t *rev_raw);
AM_ERROR am_rpi_info_clk(uint16_t *clk, uint16_t *clk_min, uint16_t *clk_max);
AM_ERROR am_rpi_info_memory(uint32_t *mem, uint32_t *used, uint32_t *free, uint32_t *free_buffers_cache);
AM_ERROR am_rpi_info_temperature(double *t);

//----------------------------------------
// miscellaneous

#if __ARM_ARCH_6__ == 1
static inline void am_rpi_delay_nop1(void)   { __asm__ volatile("NOP"); };
static inline void am_rpi_delay_nop2(void)   { __asm__ volatile("NOP\nNOP"); };
static inline void am_rpi_delay_nop3(void)   { __asm__ volatile("NOP\nNOP\nNOP"); };
static inline void am_rpi_delay_nop4(void)   { __asm__ volatile("NOP\nNOP\nNOP\nNOP"); };
static inline void am_rpi_delay_nop5(void)   { __asm__ volatile("NOP\nNOP\nNOP\nNOP\nNOP"); };
static inline void am_rpi_delay_nop6(void)   { __asm__ volatile("NOP\nNOP\nNOP\nNOP\nNOP\nNOP"); };
static inline void am_rpi_delay_nop7(void)   { __asm__ volatile("NOP\nNOP\nNOP\nNOP\nNOP\nNOP\nNOP"); };
static inline void am_rpi_delay_nop8(void)   { __asm__ volatile("NOP\nNOP\nNOP\nNOP\nNOP\nNOP\nNOP\nNOP"); };
static inline void am_rpi_delay_nop9(void)   { __asm__ volatile("NOP\nNOP\nNOP\nNOP\nNOP\nNOP\nNOP\nNOP\nNOP"); };
static inline void am_rpi_delay_nop10(void)  { __asm__ volatile("NOP\nNOP\nNOP\nNOP\nNOP\nNOP\nNOP\nNOP\nNOP\nNOP"); };
static inline void am_rpi_delay_nop11(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop11"); };
static inline void am_rpi_delay_nop12(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop12"); };
static inline void am_rpi_delay_nop13(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop13"); };
static inline void am_rpi_delay_nop14(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop14"); };
static inline void am_rpi_delay_nop15(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop15"); };
static inline void am_rpi_delay_nop16(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop16"); };
static inline void am_rpi_delay_nop17(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop17"); };
static inline void am_rpi_delay_nop18(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop18"); };
static inline void am_rpi_delay_nop19(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop19"); };
static inline void am_rpi_delay_nop20(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop20"); };
static inline void am_rpi_delay_nop21(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop21"); };
static inline void am_rpi_delay_nop22(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop22"); };
static inline void am_rpi_delay_nop23(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop23"); };
static inline void am_rpi_delay_nop24(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop24"); };
static inline void am_rpi_delay_nop25(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop25"); };
static inline void am_rpi_delay_nop26(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop26"); };
static inline void am_rpi_delay_nop27(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop27"); };
static inline void am_rpi_delay_nop28(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop28"); };
static inline void am_rpi_delay_nop29(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop29"); };
static inline void am_rpi_delay_nop30(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop30"); };
static inline void am_rpi_delay_nop31(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop31"); };
static inline void am_rpi_delay_nop32(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop32"); };
static inline void am_rpi_delay_nop33(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop33"); };
static inline void am_rpi_delay_nop34(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop34"); };
static inline void am_rpi_delay_nop35(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop35"); };
static inline void am_rpi_delay_nop36(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop36"); };
static inline void am_rpi_delay_nop37(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop37"); };
static inline void am_rpi_delay_nop38(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop38"); };
static inline void am_rpi_delay_nop39(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop39"); };
static inline void am_rpi_delay_nop40(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop40"); };
static inline void am_rpi_delay_nop41(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop41"); };
static inline void am_rpi_delay_nop42(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop42"); };
static inline void am_rpi_delay_nop43(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop43"); };
static inline void am_rpi_delay_nop44(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop44"); };
static inline void am_rpi_delay_nop45(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop45"); };
static inline void am_rpi_delay_nop46(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop46"); };
static inline void am_rpi_delay_nop47(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop47"); };
static inline void am_rpi_delay_nop48(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop48"); };
static inline void am_rpi_delay_nop49(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop49"); };
static inline void am_rpi_delay_nop50(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop50"); };
static inline void am_rpi_delay_nop51(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop51"); };
static inline void am_rpi_delay_nop52(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop52"); };
static inline void am_rpi_delay_nop53(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop53"); };
static inline void am_rpi_delay_nop54(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop54"); };
static inline void am_rpi_delay_nop55(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop55"); };
static inline void am_rpi_delay_nop56(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop56"); };
static inline void am_rpi_delay_nop57(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop57"); };
static inline void am_rpi_delay_nop58(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop58"); };
static inline void am_rpi_delay_nop59(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop59"); };
static inline void am_rpi_delay_nop60(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop60"); };
static inline void am_rpi_delay_nop61(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop61"); };
static inline void am_rpi_delay_nop62(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop62"); };
static inline void am_rpi_delay_nop63(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop63"); };
static inline void am_rpi_delay_nop64(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop64"); };
static inline void am_rpi_delay_nop65(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop65"); };
static inline void am_rpi_delay_nop66(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop66"); };
static inline void am_rpi_delay_nop67(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop67"); };
static inline void am_rpi_delay_nop68(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop68"); };
static inline void am_rpi_delay_nop69(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop69"); };
static inline void am_rpi_delay_nop70(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop70"); };
static inline void am_rpi_delay_nop71(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop71"); };
static inline void am_rpi_delay_nop72(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop72"); };
static inline void am_rpi_delay_nop73(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop73"); };
static inline void am_rpi_delay_nop74(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop74"); };
static inline void am_rpi_delay_nop75(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop75"); };
static inline void am_rpi_delay_nop76(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop76"); };
static inline void am_rpi_delay_nop77(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop77"); };
static inline void am_rpi_delay_nop78(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop78"); };
static inline void am_rpi_delay_nop79(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop79"); };
static inline void am_rpi_delay_nop80(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop80"); };
static inline void am_rpi_delay_nop81(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop81"); };
static inline void am_rpi_delay_nop82(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop82"); };
static inline void am_rpi_delay_nop83(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop83"); };
static inline void am_rpi_delay_nop84(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop84"); };
static inline void am_rpi_delay_nop85(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop85"); };
static inline void am_rpi_delay_nop86(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop86"); };
static inline void am_rpi_delay_nop87(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop87"); };
static inline void am_rpi_delay_nop88(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop88"); };
static inline void am_rpi_delay_nop89(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop89"); };
static inline void am_rpi_delay_nop90(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop90"); };
static inline void am_rpi_delay_nop91(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop91"); };
static inline void am_rpi_delay_nop92(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop92"); };
static inline void am_rpi_delay_nop93(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop93"); };
static inline void am_rpi_delay_nop94(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop94"); };
static inline void am_rpi_delay_nop95(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop95"); };
static inline void am_rpi_delay_nop96(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop96"); };
static inline void am_rpi_delay_nop97(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop97"); };
static inline void am_rpi_delay_nop98(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop98"); };
static inline void am_rpi_delay_nop99(void)  { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop99"); };
static inline void am_rpi_delay_nop100(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop100"); };
static inline void am_rpi_delay_nop101(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop101"); };
static inline void am_rpi_delay_nop102(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop102"); };
static inline void am_rpi_delay_nop103(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop103"); };
static inline void am_rpi_delay_nop104(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop104"); };
static inline void am_rpi_delay_nop105(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop105"); };
static inline void am_rpi_delay_nop106(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop106"); };
static inline void am_rpi_delay_nop107(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop107"); };
static inline void am_rpi_delay_nop108(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop108"); };
static inline void am_rpi_delay_nop109(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop109"); };
static inline void am_rpi_delay_nop110(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop110"); };
static inline void am_rpi_delay_nop111(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop111"); };
static inline void am_rpi_delay_nop112(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop112"); };
static inline void am_rpi_delay_nop113(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop113"); };
static inline void am_rpi_delay_nop114(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop114"); };
static inline void am_rpi_delay_nop115(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop115"); };
static inline void am_rpi_delay_nop116(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop116"); };
static inline void am_rpi_delay_nop117(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop117"); };
static inline void am_rpi_delay_nop118(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop118"); };
static inline void am_rpi_delay_nop119(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop119"); };
static inline void am_rpi_delay_nop120(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop120"); };
static inline void am_rpi_delay_nop121(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop121"); };
static inline void am_rpi_delay_nop122(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop122"); };
static inline void am_rpi_delay_nop123(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop123"); };
static inline void am_rpi_delay_nop124(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop124"); };
static inline void am_rpi_delay_nop125(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop125"); };
static inline void am_rpi_delay_nop126(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop126"); };
static inline void am_rpi_delay_nop127(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop127"); };
static inline void am_rpi_delay_nop128(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop128"); };
static inline void am_rpi_delay_nop129(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop129"); };
static inline void am_rpi_delay_nop130(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop130"); };
static inline void am_rpi_delay_nop131(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop131"); };
static inline void am_rpi_delay_nop132(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop132"); };
static inline void am_rpi_delay_nop133(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop133"); };
static inline void am_rpi_delay_nop134(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop134"); };
static inline void am_rpi_delay_nop135(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop135"); };
static inline void am_rpi_delay_nop136(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop136"); };
static inline void am_rpi_delay_nop137(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop137"); };
static inline void am_rpi_delay_nop138(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop138"); };
static inline void am_rpi_delay_nop139(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop139"); };
static inline void am_rpi_delay_nop140(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop140"); };
static inline void am_rpi_delay_nop141(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop141"); };
static inline void am_rpi_delay_nop142(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop142"); };
static inline void am_rpi_delay_nop143(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop143"); };
static inline void am_rpi_delay_nop144(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop144"); };
static inline void am_rpi_delay_nop145(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop145"); };
static inline void am_rpi_delay_nop146(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop146"); };
static inline void am_rpi_delay_nop147(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop147"); };
static inline void am_rpi_delay_nop148(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop148"); };
static inline void am_rpi_delay_nop149(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop149"); };
static inline void am_rpi_delay_nop150(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop150"); };
static inline void am_rpi_delay_nop151(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop151"); };
static inline void am_rpi_delay_nop152(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop152"); };
static inline void am_rpi_delay_nop153(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop153"); };
static inline void am_rpi_delay_nop154(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop154"); };
static inline void am_rpi_delay_nop155(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop155"); };
static inline void am_rpi_delay_nop156(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop156"); };
static inline void am_rpi_delay_nop157(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop157"); };
static inline void am_rpi_delay_nop158(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop158"); };
static inline void am_rpi_delay_nop159(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop159"); };
static inline void am_rpi_delay_nop160(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop160"); };
static inline void am_rpi_delay_nop161(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop161"); };
static inline void am_rpi_delay_nop162(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop162"); };
static inline void am_rpi_delay_nop163(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop163"); };
static inline void am_rpi_delay_nop164(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop164"); };
static inline void am_rpi_delay_nop165(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop165"); };
static inline void am_rpi_delay_nop166(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop166"); };
static inline void am_rpi_delay_nop167(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop167"); };
static inline void am_rpi_delay_nop168(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop168"); };
static inline void am_rpi_delay_nop169(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop169"); };
static inline void am_rpi_delay_nop170(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop170"); };
static inline void am_rpi_delay_nop171(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop171"); };
static inline void am_rpi_delay_nop172(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop172"); };
static inline void am_rpi_delay_nop173(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop173"); };
static inline void am_rpi_delay_nop174(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop174"); };
static inline void am_rpi_delay_nop175(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop175"); };
static inline void am_rpi_delay_nop176(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop176"); };
static inline void am_rpi_delay_nop177(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop177"); };
static inline void am_rpi_delay_nop178(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop178"); };
static inline void am_rpi_delay_nop179(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop179"); };
static inline void am_rpi_delay_nop180(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop180"); };
static inline void am_rpi_delay_nop181(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop181"); };
static inline void am_rpi_delay_nop182(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop182"); };
static inline void am_rpi_delay_nop183(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop183"); };
static inline void am_rpi_delay_nop184(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop184"); };
static inline void am_rpi_delay_nop185(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop185"); };
static inline void am_rpi_delay_nop186(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop186"); };
static inline void am_rpi_delay_nop187(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop187"); };
static inline void am_rpi_delay_nop188(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop188"); };
static inline void am_rpi_delay_nop189(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop189"); };
static inline void am_rpi_delay_nop190(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop190"); };
static inline void am_rpi_delay_nop191(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop191"); };
static inline void am_rpi_delay_nop192(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop192"); };
static inline void am_rpi_delay_nop193(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop193"); };
static inline void am_rpi_delay_nop194(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop194"); };
static inline void am_rpi_delay_nop195(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop195"); };
static inline void am_rpi_delay_nop196(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop196"); };
static inline void am_rpi_delay_nop197(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop197"); };
static inline void am_rpi_delay_nop198(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop198"); };
static inline void am_rpi_delay_nop199(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop199"); };
static inline void am_rpi_delay_nop200(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop200"); };
static inline void am_rpi_delay_nop201(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop201"); };
static inline void am_rpi_delay_nop202(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop202"); };
static inline void am_rpi_delay_nop203(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop203"); };
static inline void am_rpi_delay_nop204(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop204"); };
static inline void am_rpi_delay_nop205(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop205"); };
static inline void am_rpi_delay_nop206(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop206"); };
static inline void am_rpi_delay_nop207(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop207"); };
static inline void am_rpi_delay_nop208(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop208"); };
static inline void am_rpi_delay_nop209(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop209"); };
static inline void am_rpi_delay_nop210(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop210"); };
static inline void am_rpi_delay_nop211(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop211"); };
static inline void am_rpi_delay_nop212(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop212"); };
static inline void am_rpi_delay_nop213(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop213"); };
static inline void am_rpi_delay_nop214(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop214"); };
static inline void am_rpi_delay_nop215(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop215"); };
static inline void am_rpi_delay_nop216(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop216"); };
static inline void am_rpi_delay_nop217(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop217"); };
static inline void am_rpi_delay_nop218(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop218"); };
static inline void am_rpi_delay_nop219(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop219"); };
static inline void am_rpi_delay_nop220(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop220"); };
static inline void am_rpi_delay_nop221(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop221"); };
static inline void am_rpi_delay_nop222(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop222"); };
static inline void am_rpi_delay_nop223(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop223"); };
static inline void am_rpi_delay_nop224(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop224"); };
static inline void am_rpi_delay_nop225(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop225"); };
static inline void am_rpi_delay_nop226(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop226"); };
static inline void am_rpi_delay_nop227(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop227"); };
static inline void am_rpi_delay_nop228(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop228"); };
static inline void am_rpi_delay_nop229(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop229"); };
static inline void am_rpi_delay_nop230(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop230"); };
static inline void am_rpi_delay_nop231(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop231"); };
static inline void am_rpi_delay_nop232(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop232"); };
static inline void am_rpi_delay_nop233(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop233"); };
static inline void am_rpi_delay_nop234(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop234"); };
static inline void am_rpi_delay_nop235(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop235"); };
static inline void am_rpi_delay_nop236(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop236"); };
static inline void am_rpi_delay_nop237(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop237"); };
static inline void am_rpi_delay_nop238(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop238"); };
static inline void am_rpi_delay_nop239(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop239"); };
static inline void am_rpi_delay_nop240(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop240"); };
static inline void am_rpi_delay_nop241(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop241"); };
static inline void am_rpi_delay_nop242(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop242"); };
static inline void am_rpi_delay_nop243(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop243"); };
static inline void am_rpi_delay_nop244(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop244"); };
static inline void am_rpi_delay_nop245(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop245"); };
static inline void am_rpi_delay_nop246(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop246"); };
static inline void am_rpi_delay_nop247(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop247"); };
static inline void am_rpi_delay_nop248(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop248"); };
static inline void am_rpi_delay_nop249(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop249"); };
static inline void am_rpi_delay_nop250(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop250"); };
static inline void am_rpi_delay_nop251(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop251"); };
static inline void am_rpi_delay_nop252(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop252"); };
static inline void am_rpi_delay_nop253(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop253"); };
static inline void am_rpi_delay_nop254(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop254"); };
static inline void am_rpi_delay_nop255(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop255"); };
static inline void am_rpi_delay_nop256(void) { __asm__ volatile("PUSH {PC}\nB _am_rpi_delay_nop256"); };
#else   // dummy definitions for compilation on non-ARM
static inline void am_rpi_delay_nop40() {};
static inline void am_rpi_delay_nop42() {};
static inline void am_rpi_delay_nop56() {};
static inline void am_rpi_delay_nop83() {};
static inline void am_rpi_delay_nop100() {};
static inline void am_rpi_delay_nop110() {};
static inline void am_rpi_delay_nop175() {};
static inline void am_rpi_delay_nop250() {};
#endif

void am_rpi_delay_us(uint16_t us);

//----------------------------------------
// system timer
uint64_t am_rpi_timer(void);
uint32_t am_rpi_timer_lo(void);

//----------------------------------------
// I/O

enum am_rpi_gpio {
    GPIO_0  = 1 << 0,       //CSI pin 14 (GPIO_2 on Rev. 1.0)
    GPIO_1  = 1 << 1,       //CSI pin 13 (GPIO_3 on Rev. 1.0)
    GPIO_2  = 1 << 2,       //(GPIO_0 on Rev. 1.0)
    GPIO_3  = 1 << 3,       //(GPIO_1 on Rev. 1.0)
    GPIO_4  = 1 << 4,
    GPIO_5  = 1 << 5,       //CSI pin 12
    //GPIO_6  = ,
    GPIO_7  = 1 << 7,
    GPIO_8  = 1 << 8,
    GPIO_9  = 1 << 9,
    GPIO_10 = 1 << 10,
    GPIO_11 = 1 << 11,
    //GPIO_12 = ,
    //GPIO_13 = ,
    GPIO_14 = 1 << 14,
    GPIO_15 = 1 << 15,
    //GPIO_16 = ,
    GPIO_17 = 1 << 17,
    GPIO_18 = 1 << 18,
    //GPIO_19 = ,
    //GPIO_20 = ,
    GPIO_21 = 1 << 21,      //CSI pin 11 (GPIO_27 on Rev. 1.0)
    GPIO_22 = 1 << 22,
    GPIO_23 = 1 << 23,
    GPIO_24 = 1 << 24,
    GPIO_25 = 1 << 25,
    //GPIO_26 = ,
    GPIO_27 = 1 << 27,      //(GPIO_21 on Rev. 1.0)

    GPIO_28 = 1 << 28,      //on P5 (Rev. 2.0)
    GPIO_29 = 1 << 29,      //on P5 (Rev. 2.0)
    GPIO_30 = 1 << 30,      //on P5 (Rev. 2.0)
    GPIO_31 = 1 << 31,      //on P5 (Rev. 2.0)

    GPIO_P1_3  = GPIO_2,    //(GPIO_0 on Rev. 1.0)
    GPIO_P1_5  = GPIO_3,    //(GPIO_1 on Rev. 1.0)
    GPIO_P1_7  = GPIO_4,
    GPIO_P1_8  = GPIO_14,
    GPIO_P1_10 = GPIO_15,
    GPIO_P1_11 = GPIO_17,
    GPIO_P1_12 = GPIO_18,
    GPIO_P1_13 = GPIO_27,   //(GPIO_21 on Rev. 1.0)
    GPIO_P1_15 = GPIO_22,
    GPIO_P1_16 = GPIO_23,
    GPIO_P1_18 = GPIO_24,
    GPIO_P1_19 = GPIO_10,
    GPIO_P1_21 = GPIO_9,
    GPIO_P1_22 = GPIO_25,
    GPIO_P1_23 = GPIO_11,
    GPIO_P1_24 = GPIO_8,
    GPIO_P1_26 = GPIO_7,

    GPIO_P5_3 = GPIO_28,    //(Rev. 2.0)
    GPIO_P5_4 = GPIO_29,    //(Rev. 2.0)
    GPIO_P5_5 = GPIO_30,    //(Rev. 2.0)
    GPIO_P5_6 = GPIO_31,    //(Rev. 2.0)

    GPIO_CSI_14 = GPIO_0,   //(GPIO_2 on Rev. 1.0)
    GPIO_CSI_13 = GPIO_1,   //(GPIO_3 on Rev. 1.0)
    GPIO_CSI_12 = GPIO_5,
    GPIO_CSI_11 = GPIO_21,  //(GPIO_27 on Rev. 1.0)

    // all GPIOs on P1
    GPIO_P1 = GPIO_P1_3 | GPIO_P1_5 | GPIO_P1_7 | GPIO_P1_8 | GPIO_P1_10 | GPIO_P1_11 | GPIO_P1_12 | GPIO_P1_13 | GPIO_P1_15 | GPIO_P1_16 | GPIO_P1_18 | GPIO_P1_19 | GPIO_P1_21 | GPIO_P1_22 | GPIO_P1_23 | GPIO_P1_24 | GPIO_P1_26,
    // all GPIOs on P5
    GPIO_P5 = GPIO_P5_3 | GPIO_P5_4 | GPIO_P5_5 | GPIO_P5_6,
    // all GPIO on CSI
    GPIO_CSI = GPIO_CSI_11 | GPIO_CSI_12 | GPIO_CSI_13 | GPIO_CSI_14,

#if __AM_RPI_USE_CSI_GPIO
    GPIO_ALL = GPIO_P1 | GPIO_P5 | GPIO_CSI
#else
    GPIO_ALL = GPIO_P1 | GPIO_P5
#endif
};

//---------------------
// I/O configuration

enum am_rpi_gpiomode {
    GPIO_INPUT  = 0,
    GPIO_OUTPUT = 1,
    GPIO_ALT0   = 4,
    GPIO_ALT1   = 5,
    GPIO_ALT2   = 6,
    GPIO_ALT3   = 7,
    GPIO_ALT4   = 3,
    GPIO_ALT5   = 2
};
AM_ERROR am_rpi_gpio_config(uint32_t gpios, enum am_rpi_gpiomode mode);
int      am_rpi_gpio_config_read(enum am_rpi_gpio gpio);

enum am_rpi_pullupdown {
    DISABLE   = 0,
    PULL_DOWN = 1,
    PULL_UP   = 2
};
AM_ERROR am_rpi_gpio_pullupdown(uint32_t gpios, enum am_rpi_pullupdown pullupdown);

//---------------------
// GPIO

uint32_t am_rpi_gpio_read( uint32_t gpios);
AM_ERROR am_rpi_gpio_write(uint32_t gpios, bool level);
AM_ERROR am_rpi_gpio_set(  uint32_t gpios);
AM_ERROR am_rpi_gpio_clr(  uint32_t gpios);

//---------------------
// SPI

void am_rpi_spi_pins(bool cs0, bool cs1);

AM_ERROR am_rpi_spi_lock(int32_t timeout_ms);
void     am_rpi_spi_unlock(void);
void     am_rpi_spi_locking_timeout(uint16_t timeout_ms);
void     am_rpi_spi_fakelock(void);

enum am_rpi_spi_mode {
    SPI_DONTCHANGE = -1,
    SPI_MODE_MASTER = 0,
    SPI_MODE_BIDIR = 1,
    SPI_MODE_LoSSI = 2
};
enum am_rpi_spi_clkcfg {
    //SPI_DONTCHANGE = -1,
    SPI_CLKCFG_DONTCHANGE = -1,
    SPI_CLK_IDLELOW_SHIFTFALLING_READRISING  = 0,
    SPI_CLK_IDLELOW_SHIFTRISING_READFALLING  = 1,
    SPI_CLK_IDLEHIGH_SHIFTRISING_READFALLING = 2,
    SPI_CLK_IDLEHIGH_SHIFTFALLING_READRISING = 3,
};
enum am_rpi_spi_cspol {
    //SPI_DONTCHANGE = -1,
    SPI_CS_DONTCHANGE = -1,
    SPI_CS_LOWACTIVE  = 0,
    SPI_CS_HIGHACTIVE = 1
};
AM_ERROR am_rpi_spi_config(enum am_rpi_spi_mode mode, enum am_rpi_spi_clkcfg clkcfg, enum am_rpi_spi_cspol cspol0, enum am_rpi_spi_cspol cspol1);
void     am_rpi_spi_config_read(enum am_rpi_spi_mode *mode, enum am_rpi_spi_clkcfg *clkcfg, enum am_rpi_spi_cspol *cspol0, enum am_rpi_spi_cspol *cspol1);

enum am_rpi_spi_clk {
    SPI_CLK_DONTCHANGE = -1,
    SPI_CLK_62_5MHz = 4,        // 62.5 MHz
    SPI_CLK_42MHz   = 6,        // 41.67 MHz
    SPI_CLK_41_7MHz = 6,        //
    SPI_CLK_32MHz   = 8,        // 31.25 MHz
    SPI_CLK_31_3MHz = 8,        //
    SPI_CLK_25MHz   = 10,       // 25 MHz
    SPI_CLK_20_8MHz = 12,       // 20.83 MHz
    SPI_CLK_20MHz   = 14,       // 17.857MHz
    SPI_CLK_17_9MHz = 14,       // 17.857 MHz
    SPI_CLK_15_6MHz = 16,       // 15.625 MHz
    SPI_CLK_15MHz   = 18,       // 13.89 MHz
    SPI_CLK_12_5MHz = 20,       // 12.5 MHz
    SPI_CLK_10MHz   = 26,       //  9.6154 Mhz
    SPI_CLK_9_62MHz = 26,       //
    SPI_CLK_9MHz    = 28,       //  8.929 MHz
    SPI_CLK_8MHz    = 32,       //  7.813 MHz
    SPI_CLK_7_81MHz = 32,       //  7.8125 MHz
    SPI_CLK_7MHz    = 36,       //  6.944 MHz
    SPI_CLK_6_25MHz = 40,       //  6.25 MHz
    SPI_CLK_6MHz    = 42,       //  5.952 MHz
    SPI_CLK_5MHz    = 50,       //  5.0 MHz
    SPI_CLK_4MHz    = 64,       //  3.90625 MHz
    SPI_CLK_3_91MHz = 64,       //
    SPI_CLK_3MHz    = 84,       //  2.976 MHz
    SPI_CLK_2_5MHz  = 100,      //  2.5 MHz
    SPI_CLK_2MHz    = 126,      //  1.984 MHz
    SPI_CLK_1_95MHz = 128,      //  1.953125 MHz
    SPI_CLK_1MHz    = 250,      //  1.0 MHz
    SPI_CLK_977kHz  = 256,      //    976.5625 kHz
    SPI_CLK_750kHz  = 334,      //    748.50 kHz
    SPI_CLK_500kHz  = 500,      //    500.0 kHz
    SPI_CLK_488kHz  = 512,      //    488.28125 kHz
    SPI_CLK_400kHz  = 626,      //    399.36 kHz
    SPI_CLK_300kHz  = 834,      //    299.8 kHz
    SPI_CLK_250kHz  = 1000,     //    250 kHz
    SPI_CLK_244kHz  = 1024,     //    244.140625 kHz
    SPI_CLK_200kHz  = 1250,     //    200 kHz
    SPI_CLK_125kHz  = 2000,     //    125 kHz
    SPI_CLK_122kHz  = 2048,     //    122.0703125 kHz
    SPI_CLK_100kHz  = 2500,     //    100 kHz
    SPI_CLK_75kHz   = 3334,     //     74.99 kHz
    SPI_CLK_61_0kHz = 4096,     //     61.03515625 kHz
    SPI_CLK_50kHz   = 5000,     //     50 kHz
    SPI_CLK_30_5kHz = 8192,     //     30.517578125 kHz
    SPI_CLK_25kHz   = 10000,    //     25 kHz
    SPI_CLK_20kHz   = 12500,    //     20 kHz
    SPI_CLK_15_3kHz = 16384,    //     15.2587890625 kHz
    SPI_CLK_10kHz   = 25000,    //     10 kHz
    SPI_CLK_7_63kHz = 32768,    //      7.62939453125 kHz
    SPI_CLK_5kHz    = 50000,    //      5 kHz
    SPI_CLK_3_81kHz = 65536     //      3.814697265625 kHz
};
AM_ERROR am_rpi_spi_clk(enum am_rpi_spi_clk clkdiv);
void     am_rpi_spi_clk_read(enum am_rpi_spi_clk *clkdiv);

AM_ERROR am_rpi_spi_start(int cs);
void     am_rpi_spi_stop(void);
AM_ERROR am_rpi_spi_write_byte(char byte);
AM_ERROR am_rpi_spi_read_byte( char *byte);
int      am_rpi_spi_rw(char byte);
void     am_rpi_spi_wait_done(void);
AM_ERROR am_rpi_spi_comm(int cs, unsigned int len, char const *txdata, char *rxdata);
AM_ERROR am_rpi_spi_comm_writeN(int cs, unsigned int pre_len, char const* pre_data, unsigned int repeat, unsigned int len, char const *txdata, unsigned int suf_len, char const *suf_data);
AM_ERROR am_rpi_spi_write_window(int cs, uint16_t x0, uint16_t y0, uint16_t w, uint16_t h, uint16_t rowstride, char const *txdata, bool swap);

extern AM_SPI_INTERFACE const * const am_rpi_spi;

//---------------------
// I2C

void am_rpi_i2c_pins(void);
void am_rpi_i2c_enable(bool enable);
bool am_rpi_i2c_enabled(void);

AM_ERROR am_rpi_i2c_lock(int32_t timeout_ms);
void     am_rpi_i2c_unlock(void);
void     am_rpi_i2c_locking_timeout(uint16_t timeout_ms);
void     am_rpi_i2c_fakelock(void);

enum am_rpi_i2c_clk {
    I2C_CLK_DONTCHANGE = -1,
    //I2C_CLK_125MHz = 2,
    I2C_CLK_400kHz  = 625,
    I2C_CLK_300kHz  = 834,
    I2C_CLK_200kHz  = 1250,
    I2C_CLK_100kHz  = 2500,
    I2C_CLK_50kHz   = 5000,
    I2C_CLK_25kHz   = 10000,
    I2C_CLK_20kHz   = 12500,
    I2C_CLK_10kHz   = 25000,
    I2C_CLK_5kHz    = 50000,    // not allowed for SMBus
    I2C_CLK_3_81kHz = 0         // not allowed for SMBus
};
AM_ERROR am_rpi_i2c_clk(enum am_rpi_i2c_clk clkdiv);
void     am_rpi_i2c_clk_read(enum am_rpi_i2c_clk *clkdiv);

AM_ERROR am_rpi_i2c_read_byte( char *byte);
AM_ERROR am_rpi_i2c_write_byte(char byte);
AM_ERROR am_rpi_i2c_wait_done(void);

AM_ERROR am_rpi_i2c_read(      uint8_t address, uint16_t rxlen, char *rxdata);
AM_ERROR am_rpi_i2c_write(     uint8_t address, uint16_t txlen, char const *txdata);
AM_ERROR am_rpi_i2c_write_read(uint8_t address, uint16_t txlen, char const *txdata, uint16_t rxlen, char *rxdata);

AM_ERROR am_rpi_i2c_read10(      uint16_t address, uint16_t rxlen, char *rxdata);
AM_ERROR am_rpi_i2c_write10(     uint16_t address, uint16_t txlen, char const *txdata);
AM_ERROR am_rpi_i2c_write_read10(uint16_t address, uint16_t txlen, char const *txdata, uint16_t rxlen, char *rxdata);

extern AM_I2C_INTERFACE const * const am_rpi_i2c;

AM_ERROR am_rpi_i2c_prot(uint8_t address, enum am_cmd cmd, unsigned int txlen, char const *txdata, unsigned int rxlen, unsigned int *received_len, char *rxdata);

//----------------------------------------
// UI
char const*      am_rpi_gpio2str(enum am_rpi_gpio gpio);
enum am_rpi_gpio am_rpi_str2gpio(char const *s);

//========================================

#endif

